@@ -1,56 +1,69 @@
+import re
 import gobject
 import datetime
 import dateutil.parser
 import vobject
+import operator
+import time
 from dateutil.tz import tzutc, tzlocal
+from gettext import gettext as _
 import logging
 log = logging.getLogger("modules.Google")
 
 import conduit
 import conduit.dataproviders.DataProvider as DataProvider
 import conduit.dataproviders.Image as Image
-import conduit.Utils as Utils
+import conduit.utils as Utils
 import conduit.Exceptions as Exceptions
 from conduit.datatypes import Rid
+import conduit.datatypes.Contact as Contact
 import conduit.datatypes.Event as Event
 import conduit.datatypes.Photo as Photo
 import conduit.datatypes.Video as Video
-
-from gettext import gettext as _
+import conduit.datatypes.File as File
 
 #Distributors, if you ship python gdata >= 1.0.10 then remove this line
 #and the appropriate directories
 Utils.dataprovider_add_dir_to_path(__file__)
-import atom
-import gdata
-import gdata.service
-import gdata.photos
-import gdata.photos.service    
-import gdata.calendar
-import gdata.calendar.service
+try:
+    import atom
+    import gdata
+    import gdata.service
+    import gdata.photos
+    import gdata.photos.service    
+    import gdata.calendar
+    import gdata.calendar.service
+
+    MODULES = {
+        "GoogleCalendarTwoWay" : { "type": "dataprovider" },
+        "PicasaTwoWay" :         { "type": "dataprovider" },
+        "YouTubeSource" :        { "type": "dataprovider" },    
+        "ContactsSource" :       { "type": "dataprovider" },    
+    }
+    log.info("Module Information: %s" % Utils.get_module_information(gdata, None))
+except (ImportError, AttributeError):
+    MODULES = {}
+    log.info("Google support disabled")
 
 # time format
 FORMAT_STRING = "%Y-%m-%dT%H:%M:%S"
 
-MODULES = {
-    "GoogleCalendarTwoWay" : { "type": "dataprovider" },
-    "PicasaTwoWay" :         { "type": "dataprovider" },
-    "YouTubeSource" :        { "type": "dataprovider" },       
-}
-
 class GoogleBase(object):
     def __init__(self):
         self.username = ""
         self.password = ""
         self.loggedIn = False
-
+        
     def _do_login(self):
         raise NotImplementedError
 
     def _login(self):
-        if self.loggedIn != True:
-            self._do_login()
-            self.loggedIn == True
+        if not self.loggedIn:
+            try:
+                self._do_login()
+                self.loggedIn = True
+            except:
+                self.loggedIn = False
        
     def _set_username(self, username):
         if self.username != username:
@@ -58,7 +71,9 @@ class GoogleBase(object):
             self.loggedIn = False
     
     def _set_password(self, password):
-        self.password = password
+        if self.password != password:
+            self.password = password
+            self.loggedIn = False
 
     def set_configuration(self, config):
         self._set_username(config.get("username",""))
@@ -107,6 +122,8 @@ class GoogleCalendar(object):
         return '/calendar/feeds/' + self.get_uri() + '/private/full'
 
 def convert_madness_to_datetime(inputDate):
+    log.debug('Attempting to parse the following: %s'%inputDate)
+    logging.debug('Attempting to parse the following: %s'%inputDate)
     dateStr = None
     dateDate = None
     dateDateTime = None
@@ -132,6 +149,9 @@ def convert_madness_to_datetime(inputDate):
         
     if dateDateTime is not None:
         if dateDateTime.tzinfo is not None:
+            logging.warn("returning: %s",dateDateTime)
+            ts = dateDateTime.timetuple()
+            dateDateTime = dateDateTime.fromtimestamp(time.mktime(ts))
             return dateDateTime
         elif dateTZInfo is not None:
             return dateDateTime.replace(tzinfo=dateTZInfo)
@@ -139,6 +159,7 @@ def convert_madness_to_datetime(inputDate):
             log.warn('Waring, assuming datetime ('+dateDateTime.isoformat()+') is UTC')
             return dateDateTime.replace(tzinfo=tzutc())
                 
+    
     raise TypeError('Unable to convert to datetime')
 
 def parse_google_recur(recurString, args):
@@ -246,7 +267,10 @@ class GoogleEvent(object):
     def get_mtime(self):
         #mtimes need to be naive and local
         #Shouldn't Conduit use non-naive mTimes?
-        mTimeLocal = self.mTime.astimezone(tzlocal())
+        try:
+            mTimeLocal = self.mTime.astimezone(tzlocal())
+        except ValueError:
+            mTimeLocal = self.mTime
         mTimeLocalWithoutTZ = mTimeLocal.replace(tzinfo=None)
         return mTimeLocalWithoutTZ
         
@@ -307,9 +331,13 @@ class GoogleEvent(object):
         if self.visibility is not None:
             iCalEvent.add('class').value = self.visibility
         if self.created is not None:
-            iCalEvent.add('created').value = self.created.astimezone(tzutc())    
+            try:
+                iCalEvent.add('created').value = self.created.astimezone(tzutc())
+            except ValueError: pass
         if self.mTime is not None:
-            iCalEvent.add('last-modified').value = self.mTime.astimezone(tzutc())
+            try:
+                iCalEvent.add('last-modified').value = self.mTime.astimezone(tzutc())
+            except ValueError: pass
         #iCalEvent.vevent.add('dtstamp').value = 
         if self.recurrence is not None:
             iCalEvent.add('rrule').value = self.recurrence
@@ -320,7 +348,6 @@ class GoogleEvent(object):
         returnStr = iCalEvent.serialize()
         log.debug("Created ICal Format :\n"+returnStr)
         return returnStr
-
     
 class GoogleCalendarTwoWay(GoogleBase, DataProvider.TwoWay):
 
@@ -330,7 +357,7 @@ class GoogleCalendarTwoWay(GoogleBase, DataProvider.TwoWay):
     _module_type_ = "twoway"
     _in_type_ = "event"
     _out_type_ = "event"
-    _icon_ = "contact-new"
+    _icon_ = "appointment-new"
     
     def __init__(self):
         GoogleBase.__init__(self)
@@ -523,11 +550,11 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
     def __init__(self, *args):
         GoogleBase.__init__(self)
         Image.ImageTwoWay.__init__(self)
-        self.album = ""
+        self.albumName = ""
         self.imageSize = "None"
         self.pws = gdata.photos.service.PhotosService()
         self.galbum = None
-        self.gphoto_dict = None
+        self.gphoto_dict = {}
 
     def _get_raw_photo_url(self, photoInfo):
         return photoInfo.GetMediaURL()
@@ -543,11 +570,13 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
         
     def _upload_photo (self, uploadInfo):
         try:
-            gphoto = self.pws.InsertPhotoSimple (self.galbum, uploadInfo.name, '', uploadInfo.url)
-
+            gphoto = self.pws.InsertPhotoSimple(
+                                self.galbum,
+                                uploadInfo.name,
+                                '',
+                                uploadInfo.url)
             for tag in uploadInfo.tags:
                 self.pws.InsertTag(gphoto, str(tag))
-
             return Rid(uid=gphoto.gphoto_id.text)
         except Exception, e:
             raise Exceptions.SyncronizeError("Picasa Upload Error.")
@@ -556,22 +585,23 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
         self.pws.ClientLogin(self.username, self.password)
 
     def _get_album(self):
-        configured_album = None
+        for name,album in self._get_albums():
+            if name == self.albumName:
+                log.debug("Found album %s" % self.albumName)
+                self.galbum = album
+                return
 
-        albums = self.pws.GetUserFeed().entry
-        for album in albums:
-            if album.title.text != self.album:
-                continue
-            log.debug("Found album %s" % self.album)
-            configured_album = album
-            break
-
-        if not configured_album:
-            log.debug("Creating new album %s." % self.album)
-            configured_album = self.pws.InsertAlbum (self.album, '')
-
-        self.galbum = configured_album
+        log.debug("Creating new album %s." % self.albumName)
+        self.galbum = self._create_album(self.albumName)
 
+    def _get_albums(self):
+        albums = []
+        for album in self.pws.GetUserFeed().entry:
+            albums.append(
+                    (album.title.text,  #album name
+                    album))             #album
+        return albums
+        
     def _get_photos(self):
         self.gphoto_dict = {}
         for photo in self.pws.GetFeed(self.galbum.GetPhotosUri()).entry:
@@ -586,13 +616,16 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
             import time
             return datetime(*(time.strptime(timestamp, FORMAT_STRING)[0:6]))
 
+    def _create_album(self, album_name):            
+        self.pws.InsertAlbum(album_name, '', access='private')
+
     def refresh(self):
         Image.ImageTwoWay.refresh(self)
         self._login()
         self._get_album()
-        self._get_photos()
 
     def get_all (self):
+        self._get_photos()
         return self.gphoto_dict.keys()
         
     def get (self, LUID):
@@ -621,52 +654,221 @@ class PicasaTwoWay(GoogleBase, Image.ImageTwoWay):
         """
         Configures the PicasaTwoWay
         """
-        widget = Utils.dataprovider_glade_get_widget(
+        import gobject
+        import gtk
+        def on_login_finish(*args):
+            if self.loggedIn:
+                build_album_model()
+            Utils.dialog_reset_cursor(dlg)
+                
+        def on_response(sender, responseID):
+            if responseID == gtk.RESPONSE_OK:
+                self._set_username(username.get_text())
+                self._set_password(password.get_text())
+                self.albumName = album_combo.get_active_text()
+                self.imageSize = self._resize_combobox_get_active(resizecombobox)
+
+        def login_click(button, window, usernameEntry, passwordEntry):
+            self._set_username(usernameEntry.get_text())
+            self._set_password(passwordEntry.get_text())
+            Utils.dialog_set_busy_cursor(dlg)
+            conduit.GLOBALS.syncManager.run_blocking_dataprovider_function_calls(
+                                            self,
+                                            on_login_finish,
+                                            self._login)        
+
+        def username_password_changed(sender, username, password, login_button):
+            login_button.set_sensitive(
+                            len(username.get_text()) > 0 and len(password.get_text()) > 0)
+
+        def build_album_model():
+            album_store.clear()
+            album_count = 0
+            album_iter = None
+            for name, album in self._get_albums():       
+                iter = album_store.append((name,))
+                if name == self.albumName:
+                    album_iter = iter
+                album_count += 1
+
+            if album_iter:
+                album_combo.set_active_iter(album_iter)
+            elif self.albumName:
+                album_combo.child.set_text(self.albumName)
+            elif album_count:
+                album_combo.set_active(0)
+        
+        #get widget and dialog 
+        tree = Utils.dataprovider_glade_get_widget(
                         __file__, 
                         "picasa-config.glade", 
                         "PicasaTwoWayConfigDialog")
-                        
+
         #get a whole bunch of widgets
-        username = widget.get_widget("username")
-        password = widget.get_widget("password")
-        album = widget.get_widget("album")                
+        username = tree.get_widget('username')
+        password = tree.get_widget('password')
+        album_combo = tree.get_widget('album_combobox')
+        login_button = tree.get_widget("login_button")
+        dlg = tree.get_widget("PicasaTwoWayConfigDialog")        
+
+        resizecombobox = tree.get_widget("resize_combobox")
+        self._resize_combobox_build(resizecombobox, self.imageSize)
+
+        #connect to signals
+        login_button.connect('clicked', login_click, window, username, password)
+        username.connect('changed', username_password_changed, username, password, login_button)
+        password.connect('changed', username_password_changed, username, password, login_button)
         
         #preload the widgets        
         username.set_text(self.username)
         password.set_text(self.password)
-        album.set_text (self.album)
 
-        resizecombobox = widget.get_widget("combobox1")
-        self._resize_combobox_build(resizecombobox, self.imageSize)
+        #setup album combo
+        album_store = gtk.ListStore(gobject.TYPE_STRING)
+        album_combo.set_model (album_store)
+        cell = gtk.CellRendererText()
+        album_combo.pack_start(cell, True)
+        album_combo.set_text_column(0)
         
-        dlg = widget.get_widget("PicasaTwoWayConfigDialog")
-        response = Utils.run_dialog (dlg, window)
-        if response == True:
-            self._set_username(username.get_text())
-            self._set_password(password.get_text())
-            self.album = album.get_text()
-            self.imageSize = self._resize_combobox_get_active(resizecombobox)
-        dlg.destroy()    
+        #disable album lookup if no username entered
+        enabled = len(self.username) > 0
+        login_button.set_sensitive(enabled)
+
+        # Now run the dialog 
+        Utils.run_dialog_non_blocking(dlg, on_response, window)
         
     def get_configuration(self):
         conf = GoogleBase.get_configuration(self)
         conf.update({
             "imageSize" :   self.imageSize,
-            "album"     :   self.album})
+            "album"     :   self.albumName})
         return conf
         
     def set_configuration(self, config):
         GoogleBase.set_configuration(self, config)
         self.imageSize = config.get("imageSize","None")
-        self.album = config.get("album","")
-            
+        self.albumName = config.get("album","")
+
     def is_configured (self, isSource, isTwoWay):
         if not GoogleBase.is_configured(self, isSource, isTwoWay):
             return False
-        if len(self.album) < 1:
+        if len(self.albumName) < 1:
             return False
         return True
 
+
+class ContactsSource(GoogleBase, DataProvider.DataSource):
+    """
+    Contacts GData provider
+    """
+    _name_ = _("Google Contacts")
+    _description_ = _("Sync contacts from Google")
+    _category_ = conduit.dataproviders.CATEGORY_OFFICE
+    _module_type_ = "source"
+    _out_type_ = "contact"
+    _icon_ = "contact-new"
+
+    FEED = "http://www.google.com/m8/feeds/contacts/%s/base?max-results=25000"
+
+    def __init__(self, *args):
+        GoogleBase.__init__(self)
+        DataProvider.DataSource.__init__(self)
+        self.entries = None
+        self.feed = ""
+
+    def _set_username(self, username):
+        GoogleBase._set_username(self, username)
+        self.feed = self.FEED % self.username
+
+    def _do_login(self):
+        self.service = gdata.service.GDataService(service="cp", server="www.google.com")
+        self.service.ClientLogin(self.username, self.password)
+        
+    def _get_contact(self, LUID):
+        #get the gdata contact from google
+        gdc = self.service.Get(LUID)
+        if gdc is None or len(gdc.ToString()) < 1:
+            log.warn("Error getting/parsing gdata contact")
+            return None
+            
+        #FIXME: We should not be accessing the contact vcard directly.
+        #once we know what we can get from the gdata information, we should
+        #add the appropriate methods to the contact class to allow us to
+        #set these things
+        c = Contact.Contact()
+
+        c.vcard = vobject.vCard()
+        c.vcard.add('n')
+        c.vcard.n.value = vobject.vcard.Name(given="%s"%gdc.title.text)
+        
+        c.vcard.add('fn')
+        c.vcard.fn.value = "%s"%gdc.title.text
+        #isinstance(gdc, atom.Entry)
+        ee_names = map(operator.attrgetter('tag'),gdc.extension_elements)
+        if len(gdc.extension_elements) >0:
+            for e in [e for e in ee_names if e == 'email']:
+                c.vcard.add('email')
+                c.vcard.email.value = gdc.extension_elements[ee_names.index('email')].attributes['address']
+                c.vcard.email.type_param = 'INTERNET'
+            for e in [e for e in ee_names if e == 'phoneNumber']:
+                c.vcard.add('tel')
+                c.vcard.tel.value = gdc.extension_elements[ee_names.index('phoneNumber')].text
+                c.vcard.tel.type_param = gdc.extension_elements[ee_names.index('phoneNumber')].attributes['rel'].split('#')[1]
+            for e in [e for e in ee_names if e == 'postalAddress']:
+                c.vcard.add('adr')
+                c.vcard.adr.value = vobject.vcard.Address(gdc.extension_elements[ee_names.index('postalAddress')].text)
+               # c.vcard.adr.value =
+                c.vcard.adr.type_param = gdc.extension_elements[ee_names.index('postalAddress')].attributes['rel'].split('#')[1]
+        
+        #c.vcard.add('uid').value = gdc.id.text
+        c.set_UID(LUID)
+        c.set_mtime(convert_madness_to_datetime(gdc.updated.text))
+        c.set_open_URI(gdc.link[1].href)
+        return c    
+
+    def get_all(self):
+        DataProvider.DataSource.get_all(self)
+        # we can ask the server for everything thats changed after a certain date, including deletions
+        self._login()
+        contacts = self.service.GetFeed(self.feed).entry
+        return [str(contact.id.text) for contact in contacts]
+
+    def get(self, LUID):
+        DataProvider.DataSource.get(self, LUID)
+        return self._get_contact(LUID)
+
+    def delete(self, LUID):
+        self._login()
+        self.service.Delete(LUID)
+
+    def finish(self, aborted, error, conflict):
+        DataProvider.DataSource.finish(self)
+
+    def configure(self, window):
+        """
+        Configures the PicasaTwoWay
+        """
+        widget = Utils.dataprovider_glade_get_widget(
+                        __file__, 
+                        "contacts-config.glade", 
+                        "GoogleContactsConfigDialog")
+                        
+        #get a whole bunch of widgets
+        username = widget.get_widget("username")
+        password = widget.get_widget("password")
+        
+        #preload the widgets        
+        username.set_text(self.username)
+        password.set_text(self.password)
+        
+        dlg = widget.get_widget("GoogleContactsConfigDialog")
+        response = Utils.run_dialog (dlg, window)
+        if response == True:
+            self._set_username(username.get_text())
+            self._set_password(password.get_text())
+        dlg.destroy()    
+
+
 class YouTubeSource(DataProvider.DataSource):
     """
     Downloads YouTube videos using the gdata API.
@@ -682,6 +884,7 @@ class YouTubeSource(DataProvider.DataSource):
 
     USERS_FEED = "http://gdata.youtube.com/feeds/users"
     STD_FEEDS = "http://gdata.youtube.com/feeds/standardfeeds"
+    VIDEO_NAME_RE = re.compile(r', "t": "([^"]+)"')
 
     def __init__(self, *args):
         DataProvider.DataSource.__init__(self)
@@ -756,7 +959,7 @@ class YouTubeSource(DataProvider.DataSource):
             elif self.filter_type == 1:
                 videos = self._top_rated ()
             else:
-                if self.usr_filter_type == 0:
+                if self.user_filter_type == 0:
                     videos = self._videos_upload_by (self.username)
                 else:
                     videos = self._favorite_videos (self.username)
@@ -827,20 +1030,24 @@ class YouTubeSource(DataProvider.DataSource):
 
     # Generic extract step
     def _get_flv_video_url (self, url):
-        """
-        Previous code from here [1] was no longer working, 
-        therefore we try the trick from here [2]
-        [1] http://svn.pythonfr.org/public/pythonfr/video/youtube_client.py
-        [2] http://www.abdulqabiz.com/blog/archives/general/update_getting_youtu.php
-        """
+        import urllib2
         flv_url = ''
+        doc = urllib2.urlopen(url)
+        data = doc.read()
 
-        # extract video id
-        url_splited = url.split("watch?v=")
-        video_id = url_splited[1]
+        # extract video name
+        match = YouTubeSource.VIDEO_NAME_RE.search(data)
+        if match is not None:
+            video_name = match.group(1)
 
-        flv_url = "http://cache.googlevideo.com/get_video?video_id=%s&origin=youtube"
-        flv_url = flv_url % video_id
+            # extract video id
+            url_splited = url.split("watch?v=")
+            video_id = url_splited[1]
+
+            flv_url = "http://www.youtube.com/get_video?video_id=%s&t=%s"
+            flv_url = flv_url % (video_id, video_name)
+    
+        log.debug ("FLV URL %s" % flv_url)
 
         return flv_url
 
